@@ -66,7 +66,6 @@ comparators = util.Dispatcher(uselist=True)
 def _produce_net_changes(
     autogen_context: AutogenContext, upgrade_ops: UpgradeOps
 ) -> None:
-
     connection = autogen_context.connection
     assert connection is not None
     include_schemas = autogen_context.opts.get("include_schemas", False)
@@ -145,7 +144,6 @@ def _compare_tables(
     upgrade_ops: UpgradeOps,
     autogen_context: AutogenContext,
 ) -> None:
-
     default_schema = inspector.bind.dialect.default_schema_name
 
     # tables coming from the connection will not have "schema"
@@ -212,9 +210,8 @@ def _compare_tables(
                 (inspector),
                 # fmt: on
             )
-            sqla_compat._reflect_table(inspector, t, None)
+            sqla_compat._reflect_table(inspector, t)
         if autogen_context.run_object_filters(t, tname, "table", True, None):
-
             modify_table_ops = ops.ModifyTableOps(tname, [], schema=s)
 
             comparators.dispatch("table")(
@@ -243,7 +240,7 @@ def _compare_tables(
                 _compat_autogen_column_reflect(inspector),
                 # fmt: on
             )
-            sqla_compat._reflect_table(inspector, t, None)
+            sqla_compat._reflect_table(inspector, t)
         conn_column_info[(s, tname)] = t
 
     for s, tname in sorted(existing_tables, key=lambda x: (x[0] or "", x[1])):
@@ -255,7 +252,6 @@ def _compare_tables(
         if autogen_context.run_object_filters(
             metadata_table, tname, "table", False, conn_table
         ):
-
             modify_table_ops = ops.ModifyTableOps(tname, [], schema=s)
             with _compare_columns(
                 s,
@@ -266,7 +262,6 @@ def _compare_tables(
                 autogen_context,
                 inspector,
             ):
-
                 comparators.dispatch("table")(
                     autogen_context,
                     modify_table_ops,
@@ -449,10 +444,12 @@ class _uq_constraint_sig(_constraint_sig):
     is_index = False
     is_unique = True
 
-    def __init__(self, const: UniqueConstraint) -> None:
+    def __init__(self, const: UniqueConstraint, impl: DefaultImpl) -> None:
         self.const = const
         self.name = const.name
-        self.sig = tuple(sorted([col.name for col in const.columns]))
+        self.sig = ("UNIQUE_CONSTRAINT",) + impl.create_unique_constraint_sig(
+            const
+        )
 
     @property
     def column_names(self) -> List[str]:
@@ -465,7 +462,7 @@ class _ix_constraint_sig(_constraint_sig):
     def __init__(self, const: Index, impl: DefaultImpl) -> None:
         self.const = const
         self.name = const.name
-        self.sig = impl.create_index_sig(const)
+        self.sig = ("INDEX",) + impl.create_index_sig(const)
         self.is_unique = bool(const.unique)
 
     def md_name_to_sql_name(self, context: AutogenContext) -> Optional[str]:
@@ -532,7 +529,6 @@ def _compare_indexes_and_uniques(
     conn_table: Optional[Table],
     metadata_table: Optional[Table],
 ) -> None:
-
     inspector = autogen_context.inspector
     is_create_table = conn_table is None
     is_drop_table = metadata_table is None
@@ -620,6 +616,7 @@ def _compare_indexes_and_uniques(
     # 2a. if the dialect dupes unique indexes as unique constraints
     # (mysql and oracle), correct for that
 
+    impl = autogen_context.migration_context.impl
     if unique_constraints_duplicate_unique_indexes:
         _correct_for_uq_duplicates_uix(
             conn_uniques,
@@ -627,6 +624,7 @@ def _compare_indexes_and_uniques(
             metadata_unique_constraints,
             metadata_indexes,
             autogen_context.dialect,
+            impl,
         )
 
     # 3. give the dialect a chance to omit indexes and constraints that
@@ -644,15 +642,16 @@ def _compare_indexes_and_uniques(
     # Index and UniqueConstraint so we can easily work with them
     # interchangeably
     metadata_unique_constraints_sig = {
-        _uq_constraint_sig(uq) for uq in metadata_unique_constraints
+        _uq_constraint_sig(uq, impl) for uq in metadata_unique_constraints
     }
 
-    impl = autogen_context.migration_context.impl
     metadata_indexes_sig = {
         _ix_constraint_sig(ix, impl) for ix in metadata_indexes
     }
 
-    conn_unique_constraints = {_uq_constraint_sig(uq) for uq in conn_uniques}
+    conn_unique_constraints = {
+        _uq_constraint_sig(uq, impl) for uq in conn_uniques
+    }
 
     conn_indexes_sig = {_ix_constraint_sig(ix, impl) for ix in conn_indexes}
 
@@ -807,11 +806,11 @@ def _compare_indexes_and_uniques(
         if not conn_obj.is_index and conn_obj.sig in unnamed_metadata_uniques:
             continue
         elif removed_name in doubled_constraints:
+            conn_uq, conn_idx = doubled_constraints[removed_name]
             if (
-                conn_obj.sig not in metadata_indexes_by_sig
-                and conn_obj.sig not in metadata_uniques_by_sig
+                conn_idx.sig not in metadata_indexes_by_sig
+                and conn_uq.sig not in metadata_uniques_by_sig
             ):
-                conn_uq, conn_idx = doubled_constraints[removed_name]
                 obj_removed(conn_uq)
                 obj_removed(conn_idx)
         else:
@@ -862,6 +861,7 @@ def _correct_for_uq_duplicates_uix(
     metadata_unique_constraints,
     metadata_indexes,
     dialect,
+    impl,
 ):
     # dedupe unique indexes vs. constraints, since MySQL / Oracle
     # doesn't really have unique constraints as a separate construct.
@@ -884,7 +884,7 @@ def _correct_for_uq_duplicates_uix(
     }
 
     unnamed_metadata_uqs = {
-        _uq_constraint_sig(cons).sig
+        _uq_constraint_sig(cons, impl).sig
         for name, cons in metadata_cons_names
         if name is None
     }
@@ -908,10 +908,9 @@ def _correct_for_uq_duplicates_uix(
     for overlap in uqs_dupe_indexes:
         if overlap not in metadata_uq_names:
             if (
-                _uq_constraint_sig(uqs_dupe_indexes[overlap]).sig
+                _uq_constraint_sig(uqs_dupe_indexes[overlap], impl).sig
                 not in unnamed_metadata_uqs
             ):
-
                 conn_unique_constraints.discard(uqs_dupe_indexes[overlap])
         elif overlap not in metadata_ix_names:
             conn_indexes.discard(conn_ix_names[overlap])
@@ -924,10 +923,9 @@ def _compare_nullable(
     schema: Optional[str],
     tname: Union[quoted_name, str],
     cname: Union[quoted_name, str],
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> None:
-
     metadata_col_nullable = metadata_col.nullable
     conn_col_nullable = conn_col.nullable
     alter_column_op.existing_nullable = conn_col_nullable
@@ -966,10 +964,9 @@ def _setup_autoincrement(
     schema: Optional[str],
     tname: Union[quoted_name, str],
     cname: quoted_name,
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> None:
-
     if metadata_col.table._autoincrement_column is metadata_col:
         alter_column_op.kw["autoincrement"] = True
     elif metadata_col.autoincrement is True:
@@ -985,10 +982,9 @@ def _compare_type(
     schema: Optional[str],
     tname: Union[quoted_name, str],
     cname: Union[quoted_name, str],
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> None:
-
     conn_type = conn_col.type
     alter_column_op.existing_type = conn_type
     metadata_type = metadata_col.type
@@ -1023,11 +1019,8 @@ def _compare_type(
 
 
 def _render_server_default_for_compare(
-    metadata_default: Optional[Any],
-    metadata_col: Column,
-    autogen_context: AutogenContext,
+    metadata_default: Optional[Any], autogen_context: AutogenContext
 ) -> Optional[str]:
-
     if isinstance(metadata_default, sa_schema.DefaultClause):
         if isinstance(metadata_default.arg, str):
             metadata_default = metadata_default.arg
@@ -1039,11 +1032,7 @@ def _render_server_default_for_compare(
                 )
             )
     if isinstance(metadata_default, str):
-        if metadata_col.type._type_affinity is sqltypes.String:
-            metadata_default = re.sub(r"^'|'$", "", metadata_default)
-            return f"'{metadata_default}'"
-        else:
-            return metadata_default
+        return metadata_default
     else:
         return None
 
@@ -1064,8 +1053,8 @@ def _compare_computed_default(
     schema: Optional[str],
     tname: str,
     cname: str,
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> None:
     rendered_metadata_default = str(
         cast(sa_schema.Computed, metadata_col.server_default).sqltext.compile(
@@ -1130,10 +1119,9 @@ def _compare_server_default(
     schema: Optional[str],
     tname: Union[quoted_name, str],
     cname: Union[quoted_name, str],
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> Optional[bool]:
-
     metadata_default = metadata_col.server_default
     conn_col_default = conn_col.server_default
     if conn_col_default is None and metadata_default is None:
@@ -1190,7 +1178,7 @@ def _compare_server_default(
                 )
     else:
         rendered_metadata_default = _render_server_default_for_compare(
-            metadata_default, metadata_col, autogen_context
+            metadata_default, autogen_context
         )
 
         rendered_conn_default = (
@@ -1219,10 +1207,9 @@ def _compare_column_comment(
     schema: Optional[str],
     tname: Union[quoted_name, str],
     cname: quoted_name,
-    conn_col: Column,
-    metadata_col: Column,
+    conn_col: Column[Any],
+    metadata_col: Column[Any],
 ) -> Optional[Literal[False]]:
-
     assert autogen_context.dialect is not None
     if not autogen_context.dialect.supports_comments:
         return None
@@ -1250,7 +1237,6 @@ def _compare_foreign_keys(
     conn_table: Optional[Table],
     metadata_table: Optional[Table],
 ) -> None:
-
     # if we're doing CREATE TABLE, all FKs are created
     # inline within the table def
     if conn_table is None or metadata_table is None:
@@ -1370,7 +1356,6 @@ def _compare_table_comment(
     conn_table: Optional[Table],
     metadata_table: Optional[Table],
 ) -> None:
-
     assert autogen_context.dialect is not None
     if not autogen_context.dialect.supports_comments:
         return
