@@ -1,12 +1,23 @@
+from __future__ import annotations
+
 import os
+from typing import List
+from typing import Optional
+from typing import TYPE_CHECKING
+from typing import Union
 
 from . import autogenerate as autogen
 from . import util
 from .runtime.environment import EnvironmentContext
 from .script import ScriptDirectory
 
+if TYPE_CHECKING:
+    from alembic.config import Config
+    from alembic.script.base import Script
+    from .runtime.environment import ProcessRevisionDirectiveFn
 
-def list_templates(config):
+
+def list_templates(config: Config):
     """List available templates.
 
     :param config: a :class:`.Config` object.
@@ -18,14 +29,19 @@ def list_templates(config):
         with open(
             os.path.join(config.get_template_directory(), tempname, "README")
         ) as readme:
-            synopsis = next(readme)
+            synopsis = next(readme).rstrip()
         config.print_stdout("%s - %s", tempname, synopsis)
 
     config.print_stdout("\nTemplates are used via the 'init' command, e.g.:")
     config.print_stdout("\n  alembic init --template generic ./scripts")
 
 
-def init(config, directory, template="generic", package=False):
+def init(
+    config: Config,
+    directory: str,
+    template: str = "generic",
+    package: bool = False,
+) -> None:
     """Initialize a new scripts directory.
 
     :param config: a :class:`.Config` object.
@@ -53,27 +69,32 @@ def init(config, directory, template="generic", package=False):
         raise util.CommandError("No such template %r" % template)
 
     if not os.access(directory, os.F_OK):
-        util.status(
-            "Creating directory %s" % os.path.abspath(directory),
-            os.makedirs,
-            directory,
-        )
+        with util.status(
+            f"Creating directory {os.path.abspath(directory)!r}",
+            **config.messaging_opts,
+        ):
+            os.makedirs(directory)
 
     versions = os.path.join(directory, "versions")
-    util.status(
-        "Creating directory %s" % os.path.abspath(versions),
-        os.makedirs,
-        versions,
-    )
+    with util.status(
+        f"Creating directory {os.path.abspath(versions)!r}",
+        **config.messaging_opts,
+    ):
+        os.makedirs(versions)
 
     script = ScriptDirectory(directory)
 
+    config_file: str | None = None
     for file_ in os.listdir(template_dir):
         file_path = os.path.join(template_dir, file_)
         if file_ == "alembic.ini.mako":
+            assert config.config_file_name is not None
             config_file = os.path.abspath(config.config_file_name)
             if os.access(config_file, os.F_OK):
-                util.msg("File %s already exists, skipping" % config_file)
+                util.msg(
+                    f"File {config_file!r} already exists, skipping",
+                    **config.messaging_opts,
+                )
             else:
                 script._generate_template(
                     file_path, config_file, script_location=directory
@@ -87,28 +108,31 @@ def init(config, directory, template="generic", package=False):
             os.path.join(os.path.abspath(directory), "__init__.py"),
             os.path.join(os.path.abspath(versions), "__init__.py"),
         ]:
-            file_ = util.status("Adding %s" % path, open, path, "w")
-            file_.close()
+            with util.status("Adding {path!r}", **config.messaging_opts):
+                with open(path, "w"):
+                    pass
 
+    assert config_file is not None
     util.msg(
         "Please edit configuration/connection/logging "
-        "settings in %r before proceeding." % config_file
+        f"settings in {config_file!r} before proceeding.",
+        **config.messaging_opts,
     )
 
 
 def revision(
-    config,
-    message=None,
-    autogenerate=False,
-    sql=False,
-    head="head",
-    splice=False,
-    branch_label=None,
-    version_path=None,
-    rev_id=None,
-    depends_on=None,
-    process_revision_directives=None,
-):
+    config: Config,
+    message: Optional[str] = None,
+    autogenerate: bool = False,
+    sql: bool = False,
+    head: str = "head",
+    splice: bool = False,
+    branch_label: Optional[str] = None,
+    version_path: Optional[str] = None,
+    rev_id: Optional[str] = None,
+    depends_on: Optional[str] = None,
+    process_revision_directives: Optional[ProcessRevisionDirectiveFn] = None,
+) -> Union[Optional[Script], List[Optional[Script]]]:
     """Create a new revision file.
 
     :param config: a :class:`.Config` object.
@@ -152,8 +176,6 @@ def revision(
      the other parameters, this option is only available via programmatic
      use of :func:`.command.revision`
 
-     .. versionadded:: 0.9.0
-
     """
 
     script_directory = ScriptDirectory.from_config(config)
@@ -225,10 +247,70 @@ def revision(
         return scripts
 
 
-def merge(config, revisions, message=None, branch_label=None, rev_id=None):
-    """Merge two revisions together.  Creates a new migration file.
+def check(
+    config: "Config",
+) -> None:
+    """Check if revision command with autogenerate has pending upgrade ops.
 
-    .. versionadded:: 0.7.0
+    :param config: a :class:`.Config` object.
+
+    .. versionadded:: 1.9.0
+
+    """
+
+    script_directory = ScriptDirectory.from_config(config)
+
+    command_args = dict(
+        message=None,
+        autogenerate=True,
+        sql=False,
+        head="head",
+        splice=False,
+        branch_label=None,
+        version_path=None,
+        rev_id=None,
+        depends_on=None,
+    )
+    revision_context = autogen.RevisionContext(
+        config,
+        script_directory,
+        command_args,
+    )
+
+    def retrieve_migrations(rev, context):
+        revision_context.run_autogenerate(rev, context)
+        return []
+
+    with EnvironmentContext(
+        config,
+        script_directory,
+        fn=retrieve_migrations,
+        as_sql=False,
+        template_args=revision_context.template_args,
+        revision_context=revision_context,
+    ):
+        script_directory.run_env()
+
+    # the revision_context now has MigrationScript structure(s) present.
+
+    migration_script = revision_context.generated_revisions[-1]
+    diffs = migration_script.upgrade_ops.as_diffs()
+    if diffs:
+        raise util.AutogenerateDiffsDetected(
+            f"New upgrade operations detected: {diffs}"
+        )
+    else:
+        config.print_stdout("No new upgrade operations detected.")
+
+
+def merge(
+    config: Config,
+    revisions: str,
+    message: Optional[str] = None,
+    branch_label: Optional[str] = None,
+    rev_id: Optional[str] = None,
+) -> Optional[Script]:
+    """Merge two revisions together.  Creates a new migration file.
 
     :param config: a :class:`.Config` instance
 
@@ -256,11 +338,16 @@ def merge(config, revisions, message=None, branch_label=None, rev_id=None):
         refresh=True,
         head=revisions,
         branch_labels=branch_label,
-        **template_args
+        **template_args,  # type:ignore[arg-type]
     )
 
 
-def upgrade(config, revision, sql=False, tag=None):
+def upgrade(
+    config: Config,
+    revision: str,
+    sql: bool = False,
+    tag: Optional[str] = None,
+) -> None:
     """Upgrade to a later version.
 
     :param config: a :class:`.Config` instance.
@@ -298,7 +385,12 @@ def upgrade(config, revision, sql=False, tag=None):
         script.run_env()
 
 
-def downgrade(config, revision, sql=False, tag=None):
+def downgrade(
+    config: Config,
+    revision: str,
+    sql: bool = False,
+    tag: Optional[str] = None,
+) -> None:
     """Revert to a previous version.
 
     :param config: a :class:`.Config` instance.
@@ -364,7 +456,12 @@ def show(config, rev):
             config.print_stdout(sc.log_entry)
 
 
-def history(config, rev_range=None, verbose=False, indicate_current=False):
+def history(
+    config: Config,
+    rev_range: Optional[str] = None,
+    verbose: bool = False,
+    indicate_current: bool = False,
+) -> None:
     """List changeset scripts in chronological order.
 
     :param config: a :class:`.Config` instance.
@@ -375,10 +472,9 @@ def history(config, rev_range=None, verbose=False, indicate_current=False):
 
     :param indicate_current: indicate current revision.
 
-     ..versionadded:: 0.9.9
-
     """
-
+    base: Optional[str]
+    head: Optional[str]
     script = ScriptDirectory.from_config(config)
     if rev_range is not None:
         if ":" not in rev_range:
@@ -398,7 +494,6 @@ def history(config, rev_range=None, verbose=False, indicate_current=False):
         for sc in script.walk_revisions(
             base=base or "base", head=head or "heads"
         ):
-
             if indicate_current:
                 sc._db_current_indicator = sc.revision in currents
 
@@ -484,22 +579,17 @@ def branches(config, verbose=False):
             )
 
 
-def current(config, verbose=False, head_only=False):
+def current(config: Config, verbose: bool = False) -> None:
     """Display the current revision for a database.
 
     :param config: a :class:`.Config` instance.
 
     :param verbose: output in verbose mode.
 
-    :param head_only: deprecated; use ``verbose`` for additional output.
-
     """
 
     script = ScriptDirectory.from_config(config)
 
-    if head_only:
-        util.warn("--head-only is deprecated", stacklevel=3)
-
     def display_version(rev, context):
         if verbose:
             config.print_stdout(
@@ -511,17 +601,32 @@ def current(config, verbose=False, head_only=False):
 
         return []
 
-    with EnvironmentContext(config, script, fn=display_version):
+    with EnvironmentContext(
+        config, script, fn=display_version, dont_mutate=True
+    ):
         script.run_env()
 
 
-def stamp(config, revisions, sql=False, tag=None, purge=False):
+def stamp(
+    config: Config,
+    revision: str,
+    sql: bool = False,
+    tag: Optional[str] = None,
+    purge: bool = False,
+) -> None:
     """'stamp' the revision table with the given revision; don't
     run any migrations.
 
     :param config: a :class:`.Config` instance.
 
-    :param revision: target revision.
+    :param revision: target revision or list of revisions.   May be a list
+     to indicate stamping of multiple branch heads.
+
+     .. note:: this parameter is called "revisions" in the command line
+        interface.
+
+     .. versionchanged:: 1.2  The revision may be a single revision or
+        list of revisions when stamping multiple branch heads.
 
     :param sql: use ``--sql`` mode
 
@@ -540,9 +645,9 @@ def stamp(config, revisions, sql=False, tag=None, purge=False):
     if sql:
         destination_revs = []
         starting_rev = None
-        for revision in util.to_list(revisions):
-            if ":" in revision:
-                srev, revision = revision.split(":", 2)
+        for _revision in util.to_list(revision):
+            if ":" in _revision:
+                srev, _revision = _revision.split(":", 2)
 
                 if starting_rev != srev:
                     if starting_rev is None:
@@ -552,9 +657,9 @@ def stamp(config, revisions, sql=False, tag=None, purge=False):
                             "Stamp operation with --sql only supports a "
                             "single starting revision at a time"
                         )
-            destination_revs.append(revision)
+            destination_revs.append(_revision)
     else:
-        destination_revs = util.to_list(revisions)
+        destination_revs = util.to_list(revision)
 
     def do_stamp(rev, context):
         return script._stamp_revs(util.to_tuple(destination_revs), rev)
@@ -572,7 +677,7 @@ def stamp(config, revisions, sql=False, tag=None, purge=False):
         script.run_env()
 
 
-def edit(config, rev):
+def edit(config: Config, rev: str) -> None:
     """Edit revision script(s) using $EDITOR.
 
     :param config: a :class:`.Config` instance.
@@ -589,7 +694,7 @@ def edit(config, rev):
             if not rev:
                 raise util.CommandError("No current revisions")
             for sc in script.get_revisions(rev):
-                util.edit(sc.path)
+                util.open_in_editor(sc.path)
             return []
 
         with EnvironmentContext(config, script, fn=edit_current):
@@ -601,4 +706,31 @@ def edit(config, rev):
                 "No revision files indicated by symbol '%s'" % rev
             )
         for sc in revs:
-            util.edit(sc.path)
+            assert sc
+            util.open_in_editor(sc.path)
+
+
+def ensure_version(config: Config, sql: bool = False) -> None:
+    """Create the alembic version table if it doesn't exist already .
+
+    :param config: a :class:`.Config` instance.
+
+    :param sql: use ``--sql`` mode
+
+     .. versionadded:: 1.7.6
+
+    """
+
+    script = ScriptDirectory.from_config(config)
+
+    def do_ensure_version(rev, context):
+        context._ensure_version_table()
+        return []
+
+    with EnvironmentContext(
+        config,
+        script,
+        fn=do_ensure_version,
+        as_sql=sql,
+    ):
+        script.run_env()
