@@ -6,10 +6,16 @@ from configparser import ConfigParser
 import inspect
 import os
 import sys
+from typing import Any
+from typing import cast
 from typing import Dict
+from typing import Mapping
 from typing import Optional
 from typing import overload
 from typing import TextIO
+from typing import Union
+
+from typing_extensions import TypedDict
 
 from . import __version__
 from . import command
@@ -18,7 +24,6 @@ from .util import compat
 
 
 class Config:
-
     r"""Represent an Alembic configuration.
 
     Within an ``env.py`` script, this is available
@@ -93,12 +98,12 @@ class Config:
 
     def __init__(
         self,
-        file_: Optional[str] = None,
+        file_: Union[str, os.PathLike[str], None] = None,
         ini_section: str = "alembic",
         output_buffer: Optional[TextIO] = None,
         stdout: TextIO = sys.stdout,
         cmd_opts: Optional[Namespace] = None,
-        config_args: util.immutabledict = util.immutabledict(),
+        config_args: Mapping[str, Any] = util.immutabledict(),
         attributes: Optional[dict] = None,
     ) -> None:
         """Construct a new :class:`.Config`"""
@@ -123,7 +128,7 @@ class Config:
 
     """
 
-    config_file_name: Optional[str] = None
+    config_file_name: Union[str, os.PathLike[str], None] = None
     """Filesystem path to the .ini file in use."""
 
     config_ini_section: str = None  # type:ignore[assignment]
@@ -161,6 +166,8 @@ class Config:
         those arguments will formatted against the provided text,
         otherwise we simply output the provided text verbatim.
 
+        This is a no-op when the``quiet`` messaging option is enabled.
+
         e.g.::
 
             >>> config.print_stdout('Some text %s', 'arg')
@@ -173,7 +180,7 @@ class Config:
         else:
             output = str(text)
 
-        util.write_outstream(self.stdout, output, "\n")
+        util.write_outstream(self.stdout, output, "\n", **self.messaging_opts)
 
     @util.memoized_property
     def file_config(self):
@@ -210,6 +217,15 @@ class Config:
         package_dir = os.path.abspath(os.path.dirname(alembic.__file__))
         return os.path.join(package_dir, "templates")
 
+    @overload
+    def get_section(
+        self, name: str, default: None = ...
+    ) -> Optional[Dict[str, str]]:
+        ...
+
+    # "default" here could also be a TypeVar
+    # _MT = TypeVar("_MT", bound=Mapping[str, str]),
+    # however mypy wasn't handling that correctly (pyright was)
     @overload
     def get_section(
         self, name: str, default: Dict[str, str]
@@ -218,14 +234,19 @@ class Config:
 
     @overload
     def get_section(
-        self, name: str, default: Optional[Dict[str, str]] = ...
-    ) -> Optional[Dict[str, str]]:
+        self, name: str, default: Mapping[str, str]
+    ) -> Union[Dict[str, str], Mapping[str, str]]:
         ...
 
-    def get_section(self, name: str, default=None):
+    def get_section(
+        self, name: str, default: Optional[Mapping[str, str]] = None
+    ) -> Optional[Mapping[str, str]]:
         """Return all the configuration options from a given .ini file section
         as a dictionary.
 
+        If the given section does not exist, the value of ``default``
+        is returned, which is expected to be a dictionary or other mapping.
+
         """
         if not self.file_config.has_section(name):
             return default
@@ -310,6 +331,20 @@ class Config:
         """
         return self.get_section_option(self.config_ini_section, name, default)
 
+    @util.memoized_property
+    def messaging_opts(self) -> MessagingOptions:
+        """The messaging options."""
+        return cast(
+            MessagingOptions,
+            util.immutabledict(
+                {"quiet": getattr(self.cmd_opts, "quiet", False)}
+            ),
+        )
+
+
+class MessagingOptions(TypedDict, total=False):
+    quiet: bool
+
 
 class CommandLine:
     def __init__(self, prog: Optional[str] = None) -> None:
@@ -511,6 +546,12 @@ class CommandLine:
             action="store_true",
             help="Raise a full stack trace on error",
         )
+        parser.add_argument(
+            "-q",
+            "--quiet",
+            action="store_true",
+            help="Do not log to std output.",
+        )
         subparsers = parser.add_subparsers()
 
         positional_translations = {command.stamp: {"revision": "revisions"}}
@@ -521,7 +562,6 @@ class CommandLine:
                 and fn.__name__[0] != "_"
                 and fn.__module__ == "alembic.command"
             ):
-
                 spec = compat.inspect_getfullargspec(fn)
                 if spec[3] is not None:
                     positional = spec[0][1 : -len(spec[3])]
@@ -567,7 +607,7 @@ class CommandLine:
             if options.raiseerr:
                 raise
             else:
-                util.err(str(e))
+                util.err(str(e), **config.messaging_opts)
 
     def main(self, argv=None):
         options = self.parser.parse_args(argv)
