@@ -12,7 +12,6 @@ from typing import TYPE_CHECKING
 from typing import Union
 
 from sqlalchemy import Column
-from sqlalchemy import Index
 from sqlalchemy import literal_column
 from sqlalchemy import Numeric
 from sqlalchemy import text
@@ -50,6 +49,8 @@ from ..util import sqla_compat
 if TYPE_CHECKING:
     from typing import Literal
 
+    from sqlalchemy import Index
+    from sqlalchemy import UniqueConstraint
     from sqlalchemy.dialects.postgresql.array import ARRAY
     from sqlalchemy.dialects.postgresql.base import PGDDLCompiler
     from sqlalchemy.dialects.postgresql.hstore import HSTORE
@@ -79,18 +80,19 @@ class PostgresqlImpl(DefaultImpl):
     )
     identity_attrs_ignore = ("on_null", "order")
 
-    def create_index(self, index):
+    def create_index(self, index: Index, **kw: Any) -> None:
         # this likely defaults to None if not present, so get()
         # should normally not return the default value.  being
         # defensive in any case
         postgresql_include = index.kwargs.get("postgresql_include", None) or ()
         for col in postgresql_include:
-            if col not in index.table.c:
-                index.table.append_column(Column(col, sqltypes.NullType))
-        self._exec(CreateIndex(index))
+            if col not in index.table.c:  # type: ignore[union-attr]
+                index.table.append_column(  # type: ignore[union-attr]
+                    Column(col, sqltypes.NullType)
+                )
+        self._exec(CreateIndex(index, **kw))
 
     def prep_table_for_batch(self, batch_impl, table):
-
         for constraint in table.constraints:
             if (
                 constraint.name is not None
@@ -157,7 +159,6 @@ class PostgresqlImpl(DefaultImpl):
         existing_autoincrement: Optional[bool] = None,
         **kw: Any,
     ) -> None:
-
         using = kw.pop("postgresql_using", None)
 
         if using is not None and type_ is None:
@@ -239,7 +240,6 @@ class PostgresqlImpl(DefaultImpl):
         metadata_unique_constraints,
         metadata_indexes,
     ):
-
         doubled_constraints = {
             index
             for index in conn_indexes
@@ -308,6 +308,21 @@ class PostgresqlImpl(DefaultImpl):
                 break
         return to_remove
 
+    def _dialect_sig(
+        self, item: Union[Index, UniqueConstraint]
+    ) -> Tuple[Any, ...]:
+        if (
+            item.dialect_kwargs.get("postgresql_nulls_not_distinct")
+            is not None
+        ):
+            return (
+                (
+                    "nulls_not_distinct",
+                    item.dialect_kwargs["postgresql_nulls_not_distinct"],
+                ),
+            )
+        return ()
+
     def create_index_sig(self, index: Index) -> Tuple[Any, ...]:
         return tuple(
             self._cleanup_index_expr(
@@ -319,7 +334,14 @@ class PostgresqlImpl(DefaultImpl):
                 ),
             )
             for e in index.expressions
-        )
+        ) + self._dialect_sig(index)
+
+    def create_unique_constraint_sig(
+        self, const: UniqueConstraint
+    ) -> Tuple[Any, ...]:
+        return tuple(
+            sorted([col.name for col in const.columns])
+        ) + self._dialect_sig(const)
 
     def _compile_element(self, element: ClauseElement) -> str:
         return element.compile(
@@ -638,7 +660,6 @@ def _render_inline_exclude_constraint(
 
 
 def _postgresql_autogenerate_prefix(autogen_context: AutogenContext) -> str:
-
     imports = autogen_context.imports
     if imports is not None:
         imports.add("from sqlalchemy.dialects import postgresql")
