@@ -1,11 +1,16 @@
+from __future__ import annotations
+
 from contextlib import contextmanager
+from contextlib import nullcontext
 import logging
 import sys
 from typing import Any
+from typing import Callable
 from typing import cast
 from typing import Collection
 from typing import ContextManager
 from typing import Dict
+from typing import Iterable
 from typing import Iterator
 from typing import List
 from typing import Optional
@@ -31,14 +36,17 @@ from ..util.compat import EncodedIO
 
 if TYPE_CHECKING:
     from sqlalchemy.engine import Dialect
+    from sqlalchemy.engine import URL
     from sqlalchemy.engine.base import Connection
     from sqlalchemy.engine.base import Transaction
     from sqlalchemy.engine.mock import MockConnection
+    from sqlalchemy.sql.elements import ClauseElement
 
     from .environment import EnvironmentContext
     from ..config import Config
     from ..script.base import Script
     from ..script.base import ScriptDirectory
+    from ..script.revision import _RevisionOrBase
     from ..script.revision import Revision
     from ..script.revision import RevisionMap
 
@@ -46,11 +54,11 @@ log = logging.getLogger(__name__)
 
 
 class _ProxyTransaction:
-    def __init__(self, migration_context: "MigrationContext") -> None:
+    def __init__(self, migration_context: MigrationContext) -> None:
         self.migration_context = migration_context
 
     @property
-    def _proxied_transaction(self) -> Optional["Transaction"]:
+    def _proxied_transaction(self) -> Optional[Transaction]:
         return self.migration_context._transaction
 
     def rollback(self) -> None:
@@ -65,10 +73,10 @@ class _ProxyTransaction:
         t.commit()
         self.migration_context._transaction = None
 
-    def __enter__(self) -> "_ProxyTransaction":
+    def __enter__(self) -> _ProxyTransaction:
         return self
 
-    def __exit__(self, type_: None, value: None, traceback: None) -> None:
+    def __exit__(self, type_: Any, value: Any, traceback: Any) -> None:
         if self._proxied_transaction is not None:
             self._proxied_transaction.__exit__(type_, value, traceback)
             self.migration_context._transaction = None
@@ -91,6 +99,7 @@ class MigrationContext:
 
         # from within env.py script
         from alembic import context
+
         migration_context = context.get_context()
 
     For usage outside of an ``env.py`` script, such as for
@@ -116,6 +125,7 @@ class MigrationContext:
 
         # in any application, outside of the normal Alembic environment
         from alembic.operations import Operations
+
         op = Operations(context)
         op.alter_column("mytable", "somecolumn", nullable=True)
 
@@ -123,22 +133,22 @@ class MigrationContext:
 
     def __init__(
         self,
-        dialect: "Dialect",
-        connection: Optional["Connection"],
+        dialect: Dialect,
+        connection: Optional[Connection],
         opts: Dict[str, Any],
-        environment_context: Optional["EnvironmentContext"] = None,
+        environment_context: Optional[EnvironmentContext] = None,
     ) -> None:
         self.environment_context = environment_context
         self.opts = opts
         self.dialect = dialect
-        self.script: Optional["ScriptDirectory"] = opts.get("script")
+        self.script: Optional[ScriptDirectory] = opts.get("script")
         as_sql: bool = opts.get("as_sql", False)
         transactional_ddl = opts.get("transactional_ddl")
         self._transaction_per_migration = opts.get(
             "transaction_per_migration", False
         )
         self.on_version_apply_callbacks = opts.get("on_version_apply", ())
-        self._transaction: Optional["Transaction"] = None
+        self._transaction: Optional[Transaction] = None
 
         if as_sql:
             self.connection = cast(
@@ -152,7 +162,9 @@ class MigrationContext:
                 sqla_compat._get_connection_in_transaction(connection)
             )
 
-        self._migrations_fn = opts.get("fn")
+        self._migrations_fn: Optional[
+            Callable[..., Iterable[RevisionStep]]
+        ] = opts.get("fn")
         self.as_sql = as_sql
 
         self.purge = opts.get("purge", False)
@@ -211,14 +223,14 @@ class MigrationContext:
     @classmethod
     def configure(
         cls,
-        connection: Optional["Connection"] = None,
-        url: Optional[str] = None,
+        connection: Optional[Connection] = None,
+        url: Optional[Union[str, URL]] = None,
         dialect_name: Optional[str] = None,
-        dialect: Optional["Dialect"] = None,
-        environment_context: Optional["EnvironmentContext"] = None,
+        dialect: Optional[Dialect] = None,
+        environment_context: Optional[EnvironmentContext] = None,
         dialect_opts: Optional[Dict[str, str]] = None,
         opts: Optional[Any] = None,
-    ) -> "MigrationContext":
+    ) -> MigrationContext:
         """Create a new :class:`.MigrationContext`.
 
         This is a factory method usually called
@@ -335,11 +347,19 @@ class MigrationContext:
             self.connection = (
                 self.impl.connection
             ) = base_connection.execution_options(isolation_level="AUTOCOMMIT")
+
+            # sqlalchemy future mode will "autobegin" in any case, so take
+            # control of that "transaction" here
+            fake_trans: Optional[Transaction] = self.connection.begin()
+        else:
+            fake_trans = None
         try:
             yield
         finally:
             if not self.as_sql:
                 assert self.connection is not None
+                if fake_trans is not None:
+                    fake_trans.commit()
                 self.connection.execution_options(
                     isolation_level=current_level
                 )
@@ -354,7 +374,7 @@ class MigrationContext:
 
     def begin_transaction(
         self, _per_migration: bool = False
-    ) -> Union["_ProxyTransaction", ContextManager]:
+    ) -> Union[_ProxyTransaction, ContextManager[None]]:
         """Begin a logical transaction for migration operations.
 
         This method is used within an ``env.py`` script to demarcate where
@@ -396,12 +416,8 @@ class MigrationContext:
 
         """
 
-        @contextmanager
-        def do_nothing():
-            yield
-
         if self._in_external_transaction:
-            return do_nothing()
+            return nullcontext()
 
         if self.impl.transactional_ddl:
             transaction_now = _per_migration == self._transaction_per_migration
@@ -409,13 +425,13 @@ class MigrationContext:
             transaction_now = _per_migration is True
 
         if not transaction_now:
-            return do_nothing()
+            return nullcontext()
 
         elif not self.impl.transactional_ddl:
             assert _per_migration
 
             if self.as_sql:
-                return do_nothing()
+                return nullcontext()
             else:
                 # track our own notion of a "transaction block", which must be
                 # committed when complete.   Don't rely upon whether or not the
@@ -431,7 +447,7 @@ class MigrationContext:
                 in_transaction = self._transaction is not None
 
                 if in_transaction:
-                    return do_nothing()
+                    return nullcontext()
                 else:
                     assert self.connection is not None
                     self._transaction = (
@@ -506,9 +522,8 @@ class MigrationContext:
             if start_from_rev == "base":
                 start_from_rev = None
             elif start_from_rev is not None and self.script:
-
                 start_from_rev = [
-                    self.script.get_revision(sfr).revision
+                    cast("Script", self.script.get_revision(sfr)).revision
                     for sfr in util.to_list(start_from_rev)
                     if sfr not in (None, "base")
                 ]
@@ -528,6 +543,7 @@ class MigrationContext:
 
     def _ensure_version_table(self, purge: bool = False) -> None:
         with sqla_compat._ensure_scope_for_ddl(self.connection):
+            assert self.connection is not None
             self._version.create(self.connection, checkfirst=True)
             if purge:
                 assert self.connection is not None
@@ -539,9 +555,7 @@ class MigrationContext:
             self.connection, self.version_table, self.version_table_schema
         )
 
-    def stamp(
-        self, script_directory: "ScriptDirectory", revision: str
-    ) -> None:
+    def stamp(self, script_directory: ScriptDirectory, revision: str) -> None:
         """Stamp the version table with a specific revision.
 
         This method calculates those branches to which the given revision
@@ -557,7 +571,7 @@ class MigrationContext:
         for step in script_directory._stamp_revs(revision, heads):
             head_maintainer.update_to_step(step)
 
-    def run_migrations(self, **kw) -> None:
+    def run_migrations(self, **kw: Any) -> None:
         r"""Run the migration scripts established for this
         :class:`.MigrationContext`, if any.
 
@@ -599,10 +613,10 @@ class MigrationContext:
         assert self._migrations_fn is not None
         for step in self._migrations_fn(heads, self):
             with self.begin_transaction(_per_migration=True):
-
                 if self.as_sql and not head_maintainer.heads:
                     # for offline mode, include a CREATE TABLE from
                     # the base
+                    assert self.connection is not None
                     self._version.create(self.connection)
                 log.info("Running %s", step)
                 if self.as_sql:
@@ -626,6 +640,7 @@ class MigrationContext:
                     )
 
         if self.as_sql and not head_maintainer.heads:
+            assert self.connection is not None
             self._version.drop(self.connection)
 
     def _in_connection_transaction(self) -> bool:
@@ -636,7 +651,11 @@ class MigrationContext:
         else:
             return meth()
 
-    def execute(self, sql: str, execution_options: None = None) -> None:
+    def execute(
+        self,
+        sql: Union[ClauseElement, str],
+        execution_options: Optional[dict] = None,
+    ) -> None:
         """Execute a SQL construct or string statement.
 
         The underlying execution mechanics are used, that is
@@ -648,21 +667,21 @@ class MigrationContext:
         self.impl._exec(sql, execution_options)
 
     def _stdout_connection(
-        self, connection: Optional["Connection"]
-    ) -> "MockConnection":
+        self, connection: Optional[Connection]
+    ) -> MockConnection:
         def dump(construct, *multiparams, **params):
             self.impl._exec(construct)
 
         return MockEngineStrategy.MockConnection(self.dialect, dump)
 
     @property
-    def bind(self) -> Optional["Connection"]:
+    def bind(self) -> Optional[Connection]:
         """Return the current "bind".
 
         In online mode, this is an instance of
         :class:`sqlalchemy.engine.Connection`, and is suitable
         for ad-hoc execution of any kind of usage described
-        in :ref:`sqlexpression_toplevel` as well as
+        in SQLAlchemy Core documentation as well as
         for usage with the :meth:`sqlalchemy.schema.Table.create`
         and :meth:`sqlalchemy.schema.MetaData.create_all` methods
         of :class:`~sqlalchemy.schema.Table`,
@@ -677,7 +696,7 @@ class MigrationContext:
         return self.connection
 
     @property
-    def config(self) -> Optional["Config"]:
+    def config(self) -> Optional[Config]:
         """Return the :class:`.Config` used by the current environment,
         if any."""
 
@@ -687,7 +706,7 @@ class MigrationContext:
             return None
 
     def _compare_type(
-        self, inspector_column: "Column", metadata_column: "Column"
+        self, inspector_column: Column[Any], metadata_column: Column
     ) -> bool:
         if self._user_compare_type is False:
             return False
@@ -707,12 +726,11 @@ class MigrationContext:
 
     def _compare_server_default(
         self,
-        inspector_column: "Column",
-        metadata_column: "Column",
+        inspector_column: Column[Any],
+        metadata_column: Column[Any],
         rendered_metadata_default: Optional[str],
         rendered_column_default: Optional[str],
     ) -> bool:
-
         if self._user_compare_server_default is False:
             return False
 
@@ -737,7 +755,7 @@ class MigrationContext:
 
 
 class HeadMaintainer:
-    def __init__(self, context: "MigrationContext", heads: Any) -> None:
+    def __init__(self, context: MigrationContext, heads: Any) -> None:
         self.context = context
         self.heads = set(heads)
 
@@ -760,9 +778,11 @@ class HeadMaintainer:
                 == literal_column("'%s'" % version)
             )
         )
+
         if (
             not self.context.as_sql
             and self.context.dialect.supports_sane_rowcount
+            and ret is not None
             and ret.rowcount != 1
         ):
             raise util.CommandError(
@@ -785,9 +805,11 @@ class HeadMaintainer:
                 == literal_column("'%s'" % from_)
             )
         )
+
         if (
             not self.context.as_sql
             and self.context.dialect.supports_sane_rowcount
+            and ret is not None
             and ret.rowcount != 1
         ):
             raise util.CommandError(
@@ -797,7 +819,7 @@ class HeadMaintainer:
                 % (from_, to_, self.context.version_table, ret.rowcount)
             )
 
-    def update_to_step(self, step: Union["RevisionStep", "StampStep"]) -> None:
+    def update_to_step(self, step: Union[RevisionStep, StampStep]) -> None:
         if step.should_delete_branch(self.heads):
             vers = step.delete_version_num
             log.debug("branch delete %s", vers)
@@ -852,15 +874,15 @@ class MigrationInfo:
 
     """
 
-    is_upgrade: bool = None  # type:ignore[assignment]
+    is_upgrade: bool
     """True/False: indicates whether this operation ascends or descends the
     version tree."""
 
-    is_stamp: bool = None  # type:ignore[assignment]
+    is_stamp: bool
     """True/False: indicates whether this operation is a stamp (i.e. whether
     it results in any actual database operations)."""
 
-    up_revision_id: Optional[str] = None
+    up_revision_id: Optional[str]
     """Version string corresponding to :attr:`.Revision.revision`.
 
     In the case of a stamp operation, it is advised to use the
@@ -874,10 +896,10 @@ class MigrationInfo:
 
     """
 
-    up_revision_ids: Tuple[str, ...] = None  # type:ignore[assignment]
+    up_revision_ids: Tuple[str, ...]
     """Tuple of version strings corresponding to :attr:`.Revision.revision`.
 
-    In the majority of cases, this tuple will be a single value, synonomous
+    In the majority of cases, this tuple will be a single value, synonymous
     with the scalar value of :attr:`.MigrationInfo.up_revision_id`.
     It can be multiple revision identifiers only in the case of an
     ``alembic stamp`` operation which is moving downwards from multiple
@@ -885,7 +907,7 @@ class MigrationInfo:
 
     """
 
-    down_revision_ids: Tuple[str, ...] = None  # type:ignore[assignment]
+    down_revision_ids: Tuple[str, ...]
     """Tuple of strings representing the base revisions of this migration step.
 
     If empty, this represents a root revision; otherwise, the first item
@@ -893,12 +915,12 @@ class MigrationInfo:
     from dependencies.
     """
 
-    revision_map: "RevisionMap" = None  # type:ignore[assignment]
+    revision_map: RevisionMap
     """The revision map inside of which this operation occurs."""
 
     def __init__(
         self,
-        revision_map: "RevisionMap",
+        revision_map: RevisionMap,
         is_upgrade: bool,
         is_stamp: bool,
         up_revisions: Union[str, Tuple[str, ...]],
@@ -942,7 +964,7 @@ class MigrationInfo:
         )
 
     @property
-    def up_revision(self) -> "Revision":
+    def up_revision(self) -> Optional[Revision]:
         """Get :attr:`~.MigrationInfo.up_revision_id` as
         a :class:`.Revision`.
 
@@ -950,32 +972,31 @@ class MigrationInfo:
         return self.revision_map.get_revision(self.up_revision_id)
 
     @property
-    def up_revisions(self) -> Tuple["Revision", ...]:
+    def up_revisions(self) -> Tuple[Optional[_RevisionOrBase], ...]:
         """Get :attr:`~.MigrationInfo.up_revision_ids` as a
         :class:`.Revision`."""
         return self.revision_map.get_revisions(self.up_revision_ids)
 
     @property
-    def down_revisions(self) -> Tuple["Revision", ...]:
+    def down_revisions(self) -> Tuple[Optional[_RevisionOrBase], ...]:
         """Get :attr:`~.MigrationInfo.down_revision_ids` as a tuple of
         :class:`Revisions <.Revision>`."""
         return self.revision_map.get_revisions(self.down_revision_ids)
 
     @property
-    def source_revisions(self) -> Tuple["Revision", ...]:
+    def source_revisions(self) -> Tuple[Optional[_RevisionOrBase], ...]:
         """Get :attr:`~MigrationInfo.source_revision_ids` as a tuple of
         :class:`Revisions <.Revision>`."""
         return self.revision_map.get_revisions(self.source_revision_ids)
 
     @property
-    def destination_revisions(self) -> Tuple["Revision", ...]:
+    def destination_revisions(self) -> Tuple[Optional[_RevisionOrBase], ...]:
         """Get :attr:`~MigrationInfo.destination_revision_ids` as a tuple of
         :class:`Revisions <.Revision>`."""
         return self.revision_map.get_revisions(self.destination_revision_ids)
 
 
 class MigrationStep:
-
     from_revisions_no_deps: Tuple[str, ...]
     to_revisions_no_deps: Tuple[str, ...]
     is_upgrade: bool
@@ -987,14 +1008,14 @@ class MigrationStep:
 
     @classmethod
     def upgrade_from_script(
-        cls, revision_map: "RevisionMap", script: "Script"
-    ) -> "RevisionStep":
+        cls, revision_map: RevisionMap, script: Script
+    ) -> RevisionStep:
         return RevisionStep(revision_map, script, True)
 
     @classmethod
     def downgrade_from_script(
-        cls, revision_map: "RevisionMap", script: "Script"
-    ) -> "RevisionStep":
+        cls, revision_map: RevisionMap, script: Script
+    ) -> RevisionStep:
         return RevisionStep(revision_map, script, False)
 
     @property
@@ -1023,7 +1044,7 @@ class MigrationStep:
 
 class RevisionStep(MigrationStep):
     def __init__(
-        self, revision_map: "RevisionMap", revision: "Script", is_upgrade: bool
+        self, revision_map: RevisionMap, revision: Script, is_upgrade: bool
     ) -> None:
         self.revision_map = revision_map
         self.revision = revision
@@ -1051,7 +1072,7 @@ class RevisionStep(MigrationStep):
         )
 
     @property
-    def doc(self):
+    def doc(self) -> str:
         return self.revision.doc
 
     @property
@@ -1119,12 +1140,12 @@ class RevisionStep(MigrationStep):
         other_heads = set(heads).difference(self.from_revisions)
 
         if other_heads:
-            ancestors = set(
+            ancestors = {
                 r.revision
                 for r in self.revision_map._get_ancestor_nodes(
                     self.revision_map.get_revisions(other_heads), check=False
                 )
-            )
+            }
             from_revisions = list(
                 set(self.from_revisions).difference(ancestors)
             )
@@ -1141,12 +1162,12 @@ class RevisionStep(MigrationStep):
     def _unmerge_to_revisions(self, heads: Collection[str]) -> Tuple[str, ...]:
         other_heads = set(heads).difference([self.revision.revision])
         if other_heads:
-            ancestors = set(
+            ancestors = {
                 r.revision
                 for r in self.revision_map._get_ancestor_nodes(
                     self.revision_map.get_revisions(other_heads), check=False
                 )
-            )
+            }
             return tuple(set(self.to_revisions).difference(ancestors))
         else:
             return self.to_revisions
@@ -1230,7 +1251,7 @@ class RevisionStep(MigrationStep):
         return self.revision.revision
 
     @property
-    def info(self) -> "MigrationInfo":
+    def info(self) -> MigrationInfo:
         return MigrationInfo(
             revision_map=self.revision_map,
             up_revisions=self.revision.revision,
@@ -1247,7 +1268,7 @@ class StampStep(MigrationStep):
         to_: Optional[Union[str, Collection[str]]],
         is_upgrade: bool,
         branch_move: bool,
-        revision_map: Optional["RevisionMap"] = None,
+        revision_map: Optional[RevisionMap] = None,
     ) -> None:
         self.from_: Tuple[str, ...] = util.to_tuple(from_, default=())
         self.to_: Tuple[str, ...] = util.to_tuple(to_, default=())
@@ -1256,9 +1277,9 @@ class StampStep(MigrationStep):
         self.migration_fn = self.stamp_revision
         self.revision_map = revision_map
 
-    doc = None
+    doc: Optional[str] = None
 
-    def stamp_revision(self, **kw) -> None:
+    def stamp_revision(self, **kw: Any) -> None:
         return None
 
     def __eq__(self, other):
@@ -1345,7 +1366,7 @@ class StampStep(MigrationStep):
         return len(self.to_) > 1
 
     @property
-    def info(self) -> "MigrationInfo":
+    def info(self) -> MigrationInfo:
         up, down = (
             (self.to_, self.from_)
             if self.is_upgrade
