@@ -1,27 +1,54 @@
+from __future__ import annotations
+
 from contextlib import contextmanager
 import datetime
 import os
 import re
 import shutil
-
-from dateutil import tz
+import sys
+from types import ModuleType
+from typing import Any
+from typing import cast
+from typing import Iterator
+from typing import List
+from typing import Mapping
+from typing import Optional
+from typing import Sequence
+from typing import Set
+from typing import Tuple
+from typing import TYPE_CHECKING
+from typing import Union
 
 from . import revision
 from . import write_hooks
 from .. import util
 from ..runtime import migration
-from ..util import compat
+from ..util import not_none
+
+if TYPE_CHECKING:
+    from .revision import _RevIdType
+    from .revision import Revision
+    from ..config import Config
+    from ..config import MessagingOptions
+    from ..runtime.migration import RevisionStep
+    from ..runtime.migration import StampStep
+
+try:
+    from dateutil import tz
+except ImportError:
+    tz = None  # type: ignore[assignment]
 
 _sourceless_rev_file = re.compile(r"(?!\.\#|__init__)(.*\.py)(c|o)?$")
 _only_source_rev_file = re.compile(r"(?!\.\#|__init__)(.*\.py)$")
 _legacy_rev = re.compile(r"([a-f0-9]+)\.py$")
-_mod_def_re = re.compile(r"(upgrade|downgrade)_([a-z0-9]+)")
 _slug_re = re.compile(r"\w+")
 _default_file_template = "%(rev)s_%(slug)s"
 _split_on_space_comma = re.compile(r", *|(?: +)")
 
+_split_on_space_comma_colon = re.compile(r", *|(?: +)|\:")
 
-class ScriptDirectory(object):
+
+class ScriptDirectory:
 
     """Provides operations upon an Alembic script directory.
 
@@ -44,15 +71,19 @@ class ScriptDirectory(object):
 
     def __init__(
         self,
-        dir,  # noqa
-        file_template=_default_file_template,
-        truncate_slug_length=40,
-        version_locations=None,
-        sourceless=False,
-        output_encoding="utf-8",
-        timezone=None,
-        hook_config=None,
-    ):
+        dir: str,  # noqa
+        file_template: str = _default_file_template,
+        truncate_slug_length: Optional[int] = 40,
+        version_locations: Optional[List[str]] = None,
+        sourceless: bool = False,
+        output_encoding: str = "utf-8",
+        timezone: Optional[str] = None,
+        hook_config: Optional[Mapping[str, str]] = None,
+        recursive_version_locations: bool = False,
+        messaging_opts: MessagingOptions = cast(
+            "MessagingOptions", util.EMPTY_DICT
+        ),
+    ) -> None:
         self.dir = dir
         self.file_template = file_template
         self.version_locations = version_locations
@@ -62,6 +93,8 @@ class ScriptDirectory(object):
         self.revision_map = revision.RevisionMap(self._load_revisions)
         self.timezone = timezone
         self.hook_config = hook_config
+        self.recursive_version_locations = recursive_version_locations
+        self.messaging_opts = messaging_opts
 
         if not os.access(dir, os.F_OK):
             raise util.CommandError(
@@ -71,7 +104,7 @@ class ScriptDirectory(object):
             )
 
     @property
-    def versions(self):
+    def versions(self) -> str:
         loc = self._version_locations
         if len(loc) > 1:
             raise util.CommandError("Multiple version_locations present")
@@ -88,7 +121,7 @@ class ScriptDirectory(object):
         else:
             return (os.path.abspath(os.path.join(self.dir, "versions")),)
 
-    def _load_revisions(self):
+    def _load_revisions(self) -> Iterator[Script]:
         if self.version_locations:
             paths = [
                 vers
@@ -100,22 +133,25 @@ class ScriptDirectory(object):
 
         dupes = set()
         for vers in paths:
-            for file_ in Script._list_py_dir(self, vers):
-                path = os.path.realpath(os.path.join(vers, file_))
-                if path in dupes:
+            for file_path in Script._list_py_dir(self, vers):
+                real_path = os.path.realpath(file_path)
+                if real_path in dupes:
                     util.warn(
                         "File %s loaded twice! ignoring. Please ensure "
-                        "version_locations is unique." % path
+                        "version_locations is unique." % real_path
                     )
                     continue
-                dupes.add(path)
-                script = Script._from_filename(self, vers, file_)
+                dupes.add(real_path)
+
+                filename = os.path.basename(real_path)
+                dir_name = os.path.dirname(real_path)
+                script = Script._from_filename(self, dir_name, filename)
                 if script is None:
                     continue
                 yield script
 
     @classmethod
-    def from_config(cls, config):
+    def from_config(cls, config: Config) -> ScriptDirectory:
         """Produce a new :class:`.ScriptDirectory` given a :class:`.Config`
         instance.
 
@@ -128,14 +164,58 @@ class ScriptDirectory(object):
             raise util.CommandError(
                 "No 'script_location' key " "found in configuration."
             )
-        truncate_slug_length = config.get_main_option("truncate_slug_length")
-        if truncate_slug_length is not None:
-            truncate_slug_length = int(truncate_slug_length)
+        truncate_slug_length: Optional[int]
+        tsl = config.get_main_option("truncate_slug_length")
+        if tsl is not None:
+            truncate_slug_length = int(tsl)
+        else:
+            truncate_slug_length = None
 
-        version_locations = config.get_main_option("version_locations")
-        if version_locations:
-            version_locations = _split_on_space_comma.split(version_locations)
+        version_locations_str = config.get_main_option("version_locations")
+        version_locations: Optional[List[str]]
+        if version_locations_str:
+            version_path_separator = config.get_main_option(
+                "version_path_separator"
+            )
 
+            split_on_path = {
+                None: None,
+                "space": " ",
+                "os": os.pathsep,
+                ":": ":",
+                ";": ";",
+            }
+
+            try:
+                split_char: Optional[str] = split_on_path[
+                    version_path_separator
+                ]
+            except KeyError as ke:
+                raise ValueError(
+                    "'%s' is not a valid value for "
+                    "version_path_separator; "
+                    "expected 'space', 'os', ':', ';'" % version_path_separator
+                ) from ke
+            else:
+                if split_char is None:
+                    # legacy behaviour for backwards compatibility
+                    version_locations = _split_on_space_comma.split(
+                        version_locations_str
+                    )
+                else:
+                    version_locations = [
+                        x for x in version_locations_str.split(split_char) if x
+                    ]
+        else:
+            version_locations = None
+
+        prepend_sys_path = config.get_main_option("prepend_sys_path")
+        if prepend_sys_path:
+            sys.path[:0] = list(
+                _split_on_space_comma_colon.split(prepend_sys_path)
+            )
+
+        rvl = config.get_main_option("recursive_version_locations") == "true"
         return ScriptDirectory(
             util.coerce_resource_to_filename(script_location),
             file_template=config.get_main_option(
@@ -147,31 +227,33 @@ class ScriptDirectory(object):
             version_locations=version_locations,
             timezone=config.get_main_option("timezone"),
             hook_config=config.get_section("post_write_hooks", {}),
+            recursive_version_locations=rvl,
+            messaging_opts=config.messaging_opts,
         )
 
     @contextmanager
     def _catch_revision_errors(
         self,
-        ancestor=None,
-        multiple_heads=None,
-        start=None,
-        end=None,
-        resolution=None,
-    ):
+        ancestor: Optional[str] = None,
+        multiple_heads: Optional[str] = None,
+        start: Optional[str] = None,
+        end: Optional[str] = None,
+        resolution: Optional[str] = None,
+    ) -> Iterator[None]:
         try:
             yield
         except revision.RangeNotAncestorError as rna:
             if start is None:
-                start = rna.lower
+                start = cast(Any, rna.lower)
             if end is None:
-                end = rna.upper
+                end = cast(Any, rna.upper)
             if not ancestor:
                 ancestor = (
                     "Requested range %(start)s:%(end)s does not refer to "
                     "ancestor/descendant revisions along the same branch"
                 )
             ancestor = ancestor % {"start": start, "end": end}
-            compat.raise_(util.CommandError(ancestor), from_=rna)
+            raise util.CommandError(ancestor) from rna
         except revision.MultipleHeads as mh:
             if not multiple_heads:
                 multiple_heads = (
@@ -185,17 +267,19 @@ class ScriptDirectory(object):
                 "head_arg": end or mh.argument,
                 "heads": util.format_as_comma(mh.heads),
             }
-            compat.raise_(util.CommandError(multiple_heads), from_=mh)
+            raise util.CommandError(multiple_heads) from mh
         except revision.ResolutionError as re:
             if resolution is None:
                 resolution = "Can't locate revision identified by '%s'" % (
                     re.argument
                 )
-            compat.raise_(util.CommandError(resolution), from_=re)
+            raise util.CommandError(resolution) from re
         except revision.RevisionError as err:
-            compat.raise_(util.CommandError(err.args[0]), from_=err)
+            raise util.CommandError(err.args[0]) from err
 
-    def walk_revisions(self, base="base", head="heads"):
+    def walk_revisions(
+        self, base: str = "base", head: str = "heads"
+    ) -> Iterator[Script]:
         """Iterate through all revisions.
 
         :param base: the base revision, or "base" to start from the
@@ -210,28 +294,26 @@ class ScriptDirectory(object):
             for rev in self.revision_map.iterate_revisions(
                 head, base, inclusive=True, assert_relative_length=False
             ):
-                yield rev
+                yield cast(Script, rev)
 
-    def get_revisions(self, id_):
+    def get_revisions(self, id_: _RevIdType) -> Tuple[Optional[Script], ...]:
         """Return the :class:`.Script` instance with the given rev identifier,
         symbolic name, or sequence of identifiers.
 
         """
         with self._catch_revision_errors():
-            return self.revision_map.get_revisions(id_)
-
-    def get_all_current(self, id_):
-        with self._catch_revision_errors():
-            top_revs = set(self.revision_map.get_revisions(id_))
-            top_revs.update(
-                self.revision_map._get_ancestor_nodes(
-                    list(top_revs), include_dependencies=True
-                )
+            return cast(
+                Tuple[Optional[Script], ...],
+                self.revision_map.get_revisions(id_),
             )
-            top_revs = self.revision_map._filter_into_branch_heads(top_revs)
-            return top_revs
 
-    def get_revision(self, id_):
+    def get_all_current(self, id_: Tuple[str, ...]) -> Set[Optional[Script]]:
+        with self._catch_revision_errors():
+            return cast(
+                Set[Optional[Script]], self.revision_map._get_all_current(id_)
+            )
+
+    def get_revision(self, id_: str) -> Optional[Script]:
         """Return the :class:`.Script` instance with the given rev id.
 
         .. seealso::
@@ -241,9 +323,11 @@ class ScriptDirectory(object):
         """
 
         with self._catch_revision_errors():
-            return self.revision_map.get_revision(id_)
+            return cast(Optional[Script], self.revision_map.get_revision(id_))
 
-    def as_revision_number(self, id_):
+    def as_revision_number(
+        self, id_: Optional[str]
+    ) -> Optional[Union[str, Tuple[str, ...]]]:
         """Convert a symbolic revision, i.e. 'head' or 'base', into
         an actual revision number."""
 
@@ -258,7 +342,12 @@ class ScriptDirectory(object):
         else:
             return rev[0]
 
-    def iterate_revisions(self, upper, lower):
+    def iterate_revisions(
+        self,
+        upper: Union[str, Tuple[str, ...], None],
+        lower: Union[str, Tuple[str, ...], None],
+        **kw: Any,
+    ) -> Iterator[Script]:
         """Iterate through script revisions, starting at the given
         upper revision identifier and ending at the lower.
 
@@ -274,9 +363,12 @@ class ScriptDirectory(object):
             :meth:`.RevisionMap.iterate_revisions`
 
         """
-        return self.revision_map.iterate_revisions(upper, lower)
+        return cast(
+            Iterator[Script],
+            self.revision_map.iterate_revisions(upper, lower, **kw),
+        )
 
-    def get_current_head(self):
+    def get_current_head(self) -> Optional[str]:
         """Return the current head revision.
 
         If the script directory has multiple heads
@@ -300,7 +392,7 @@ class ScriptDirectory(object):
         ):
             return self.revision_map.get_current_head()
 
-    def get_heads(self):
+    def get_heads(self) -> List[str]:
         """Return all "versioned head" revisions as strings.
 
         This is normally a list of length one,
@@ -313,7 +405,7 @@ class ScriptDirectory(object):
         """
         return list(self.revision_map.heads)
 
-    def get_base(self):
+    def get_base(self) -> Optional[str]:
         """Return the "base" revision as a string.
 
         This is the revision number of the script that
@@ -335,7 +427,7 @@ class ScriptDirectory(object):
         else:
             return None
 
-    def get_bases(self):
+    def get_bases(self) -> List[str]:
         """return all "base" revisions as strings.
 
         This is the revision number of all scripts that
@@ -344,16 +436,17 @@ class ScriptDirectory(object):
         """
         return list(self.revision_map.bases)
 
-    def _upgrade_revs(self, destination, current_rev):
+    def _upgrade_revs(
+        self, destination: str, current_rev: str
+    ) -> List[RevisionStep]:
         with self._catch_revision_errors(
             ancestor="Destination %(end)s is not a valid upgrade "
             "target from current head(s)",
             end=destination,
         ):
-            revs = self.revision_map.iterate_revisions(
+            revs = self.iterate_revisions(
                 destination, current_rev, implicit_base=True
             )
-            revs = list(revs)
             return [
                 migration.MigrationStep.upgrade_from_script(
                     self.revision_map, script
@@ -361,13 +454,15 @@ class ScriptDirectory(object):
                 for script in reversed(list(revs))
             ]
 
-    def _downgrade_revs(self, destination, current_rev):
+    def _downgrade_revs(
+        self, destination: str, current_rev: Optional[str]
+    ) -> List[RevisionStep]:
         with self._catch_revision_errors(
             ancestor="Destination %(end)s is not a valid downgrade "
             "target from current head(s)",
             end=destination,
         ):
-            revs = self.revision_map.iterate_revisions(
+            revs = self.iterate_revisions(
                 current_rev, destination, select_for_downgrade=True
             )
             return [
@@ -377,25 +472,28 @@ class ScriptDirectory(object):
                 for script in revs
             ]
 
-    def _stamp_revs(self, revision, heads):
+    def _stamp_revs(
+        self, revision: _RevIdType, heads: _RevIdType
+    ) -> List[StampStep]:
         with self._catch_revision_errors(
             multiple_heads="Multiple heads are present; please specify a "
             "single target revision"
         ):
-
-            heads = self.get_revisions(heads)
+            heads_revs = self.get_revisions(heads)
 
             steps = []
 
             if not revision:
                 revision = "base"
 
-            filtered_heads = []
+            filtered_heads: List[Script] = []
             for rev in util.to_tuple(revision):
                 if rev:
                     filtered_heads.extend(
                         self.revision_map.filter_for_lineage(
-                            heads, rev, include_dependencies=True
+                            cast(Sequence[Script], heads_revs),
+                            rev,
+                            include_dependencies=True,
                         )
                     )
             filtered_heads = util.unique_list(filtered_heads)
@@ -403,7 +501,6 @@ class ScriptDirectory(object):
             dests = self.get_revisions(revision) or [None]
 
             for dest in dests:
-
                 if dest is None:
                     # dest is 'base'.  Return a "delete branch" migration
                     # for all applicable heads.
@@ -469,7 +566,7 @@ class ScriptDirectory(object):
 
             return steps
 
-    def run_env(self):
+    def run_env(self) -> None:
         """Run the script environment.
 
         This basically runs the ``env.py`` script present
@@ -484,28 +581,33 @@ class ScriptDirectory(object):
     def env_py_location(self):
         return os.path.abspath(os.path.join(self.dir, "env.py"))
 
-    def _generate_template(self, src, dest, **kw):
-        util.status(
-            "Generating %s" % os.path.abspath(dest),
-            util.template_to_file,
-            src,
-            dest,
-            self.output_encoding,
-            **kw
-        )
+    def _generate_template(self, src: str, dest: str, **kw: Any) -> None:
+        with util.status(
+            f"Generating {os.path.abspath(dest)}", **self.messaging_opts
+        ):
+            util.template_to_file(src, dest, self.output_encoding, **kw)
 
-    def _copy_file(self, src, dest):
-        util.status(
-            "Generating %s" % os.path.abspath(dest), shutil.copy, src, dest
-        )
+    def _copy_file(self, src: str, dest: str) -> None:
+        with util.status(
+            f"Generating {os.path.abspath(dest)}", **self.messaging_opts
+        ):
+            shutil.copy(src, dest)
 
-    def _ensure_directory(self, path):
+    def _ensure_directory(self, path: str) -> None:
         path = os.path.abspath(path)
         if not os.path.exists(path):
-            util.status("Creating directory %s" % path, os.makedirs, path)
+            with util.status(
+                f"Creating directory {path}", **self.messaging_opts
+            ):
+                os.makedirs(path)
 
-    def _generate_create_date(self):
+    def _generate_create_date(self) -> datetime.datetime:
         if self.timezone is not None:
+            if tz is None:
+                raise util.CommandError(
+                    "The library 'python-dateutil' is required "
+                    "for timezone support"
+                )
             # First, assume correct capitalization
             tzinfo = tz.gettz(self.timezone)
             if tzinfo is None:
@@ -526,16 +628,16 @@ class ScriptDirectory(object):
 
     def generate_revision(
         self,
-        revid,
-        message,
-        head=None,
-        refresh=False,
-        splice=False,
-        branch_labels=None,
-        version_path=None,
-        depends_on=None,
-        **kw
-    ):
+        revid: str,
+        message: Optional[str],
+        head: Optional[str] = None,
+        refresh: bool = False,
+        splice: Optional[bool] = False,
+        branch_labels: Optional[_RevIdType] = None,
+        version_path: Optional[str] = None,
+        depends_on: Optional[_RevIdType] = None,
+        **kw: Any,
+    ) -> Optional[Script]:
         """Generate a new revision file.
 
         This runs the ``script.py.mako`` template, given
@@ -560,7 +662,7 @@ class ScriptDirectory(object):
         try:
             Script.verify_rev_id(revid)
         except revision.RevisionError as err:
-            compat.raise_(util.CommandError(err.args[0]), from_=err)
+            raise util.CommandError(err.args[0]) from err
 
         with self._catch_revision_errors(
             multiple_heads=(
@@ -569,7 +671,12 @@ class ScriptDirectory(object):
                 "or perform a merge."
             )
         ):
-            heads = self.revision_map.get_revisions(head)
+            heads = cast(
+                Tuple[Optional["Revision"], ...],
+                self.revision_map.get_revisions(head),
+            )
+            for h in heads:
+                assert h != "base"
 
         if len(set(heads)) != len(heads):
             raise util.CommandError("Duplicate head revisions specified")
@@ -578,9 +685,10 @@ class ScriptDirectory(object):
 
         if version_path is None:
             if len(self._version_locations) > 1:
-                for head in heads:
-                    if head is not None:
-                        version_path = os.path.dirname(head.path)
+                for head_ in heads:
+                    if head_ is not None:
+                        assert isinstance(head_, Script)
+                        version_path = os.path.dirname(head_.path)
                         break
                 else:
                     raise util.CommandError(
@@ -606,25 +714,28 @@ class ScriptDirectory(object):
         path = self._rev_path(version_path, revid, message, create_date)
 
         if not splice:
-            for head in heads:
-                if head is not None and not head.is_head:
+            for head_ in heads:
+                if head_ is not None and not head_.is_head:
                     raise util.CommandError(
                         "Revision %s is not a head revision; please specify "
                         "--splice to create a new branch from this revision"
-                        % head.revision
+                        % head_.revision
                     )
 
+        resolved_depends_on: Optional[List[str]]
         if depends_on:
             with self._catch_revision_errors():
-                depends_on = [
+                resolved_depends_on = [
                     dep
                     if dep in rev.branch_labels  # maintain branch labels
                     else rev.revision  # resolve partial revision identifiers
                     for rev, dep in [
-                        (self.revision_map.get_revision(dep), dep)
+                        (not_none(self.revision_map.get_revision(dep)), dep)
                         for dep in util.to_list(depends_on)
                     ]
                 ]
+        else:
+            resolved_depends_on = None
 
         self._generate_template(
             os.path.join(self.dir, "script.py.mako"),
@@ -634,11 +745,11 @@ class ScriptDirectory(object):
                 tuple(h.revision if h is not None else None for h in heads)
             ),
             branch_labels=util.to_tuple(branch_labels),
-            depends_on=revision.tuple_rev_as_scalar(depends_on),
+            depends_on=revision.tuple_rev_as_scalar(resolved_depends_on),
             create_date=create_date,
             comma=util.format_as_comma,
             message=message if message is not None else ("empty message"),
-            **kw
+            **kw,
         )
 
         post_write_hooks = self.hook_config
@@ -648,7 +759,9 @@ class ScriptDirectory(object):
         try:
             script = Script._from_path(self, path)
         except revision.RevisionError as err:
-            compat.raise_(util.CommandError(err.args[0]), from_=err)
+            raise util.CommandError(err.args[0]) from err
+        if script is None:
+            return None
         if branch_labels and not script.branch_labels:
             raise util.CommandError(
                 "Version %s specified branch_labels %s, however the "
@@ -657,11 +770,17 @@ class ScriptDirectory(object):
                 "'branch_labels' section?"
                 % (script.revision, branch_labels, script.path)
             )
-
         self.revision_map.add_revision(script)
         return script
 
-    def _rev_path(self, path, rev_id, message, create_date):
+    def _rev_path(
+        self,
+        path: str,
+        rev_id: str,
+        message: Optional[str],
+        create_date: datetime.datetime,
+    ) -> str:
+        epoch = int(create_date.timestamp())
         slug = "_".join(_slug_re.findall(message or "")).lower()
         if len(slug) > self.truncate_slug_length:
             slug = slug[: self.truncate_slug_length].rsplit("_", 1)[0] + "_"
@@ -670,6 +789,7 @@ class ScriptDirectory(object):
             % {
                 "rev": rev_id,
                 "slug": slug,
+                "epoch": epoch,
                 "year": create_date.year,
                 "month": create_date.month,
                 "day": create_date.day,
@@ -690,12 +810,12 @@ class Script(revision.Revision):
 
     """
 
-    def __init__(self, module, rev_id, path):
+    def __init__(self, module: ModuleType, rev_id: str, path: str):
         self.module = module
         self.path = path
-        super(Script, self).__init__(
+        super().__init__(
             rev_id,
-            module.down_revision,
+            module.down_revision,  # type: ignore[attr-defined]
             branch_labels=util.to_tuple(
                 getattr(module, "branch_labels", None), default=()
             ),
@@ -704,36 +824,38 @@ class Script(revision.Revision):
             ),
         )
 
-    module = None
+    module: ModuleType
     """The Python module representing the actual script itself."""
 
-    path = None
+    path: str
     """Filesystem path of the script."""
 
-    _db_current_indicator = None
+    _db_current_indicator: Optional[bool] = None
     """Utility variable which when set will cause string output to indicate
     this is a "current" version in some database"""
 
     @property
-    def doc(self):
+    def doc(self) -> str:
         """Return the docstring given in the script."""
 
         return re.split("\n\n", self.longdoc)[0]
 
     @property
-    def longdoc(self):
+    def longdoc(self) -> str:
         """Return the docstring given in the script."""
 
         doc = self.module.__doc__
         if doc:
             if hasattr(self.module, "_alembic_source_encoding"):
-                doc = doc.decode(self.module._alembic_source_encoding)
-            return doc.strip()
+                doc = doc.decode(  # type: ignore[attr-defined]
+                    self.module._alembic_source_encoding  # type: ignore[attr-defined] # noqa
+                )
+            return doc.strip()  # type: ignore[union-attr]
         else:
             return ""
 
     @property
-    def log_entry(self):
+    def log_entry(self) -> str:
         entry = "Rev: %s%s%s%s%s\n" % (
             self.revision,
             " (head)" if self.is_head else "",
@@ -780,12 +902,12 @@ class Script(revision.Revision):
 
     def _head_only(
         self,
-        include_branches=False,
-        include_doc=False,
-        include_parents=False,
-        tree_indicators=True,
-        head_indicators=True,
-    ):
+        include_branches: bool = False,
+        include_doc: bool = False,
+        include_parents: bool = False,
+        tree_indicators: bool = True,
+        head_indicators: bool = True,
+    ) -> str:
         text = self.revision
         if include_parents:
             if self.dependencies:
@@ -796,6 +918,7 @@ class Script(revision.Revision):
                 )
             else:
                 text = "%s -> %s" % (self._format_down_revision(), text)
+        assert text is not None
         if include_branches and self.branch_labels:
             text += " (%s)" % util.format_as_comma(self.branch_labels)
         if head_indicators or tree_indicators:
@@ -817,12 +940,12 @@ class Script(revision.Revision):
 
     def cmd_format(
         self,
-        verbose,
-        include_branches=False,
-        include_doc=False,
-        include_parents=False,
-        tree_indicators=True,
-    ):
+        verbose: bool,
+        include_branches: bool = False,
+        include_doc: bool = False,
+        include_parents: bool = False,
+        tree_indicators: bool = True,
+    ) -> str:
         if verbose:
             return self.log_entry
         else:
@@ -830,42 +953,60 @@ class Script(revision.Revision):
                 include_branches, include_doc, include_parents, tree_indicators
             )
 
-    def _format_down_revision(self):
+    def _format_down_revision(self) -> str:
         if not self.down_revision:
             return "<base>"
         else:
             return util.format_as_comma(self._versioned_down_revisions)
 
     @classmethod
-    def _from_path(cls, scriptdir, path):
+    def _from_path(
+        cls, scriptdir: ScriptDirectory, path: str
+    ) -> Optional[Script]:
         dir_, filename = os.path.split(path)
         return cls._from_filename(scriptdir, dir_, filename)
 
     @classmethod
-    def _list_py_dir(cls, scriptdir, path):
-        if scriptdir.sourceless:
-            # read files in version path, e.g. pyc or pyo files
-            # in the immediate path
-            paths = os.listdir(path)
+    def _list_py_dir(cls, scriptdir: ScriptDirectory, path: str) -> List[str]:
+        paths = []
+        for root, dirs, files in os.walk(path, topdown=True):
+            if root.endswith("__pycache__"):
+                # a special case - we may include these files
+                # if a `sourceless` option is specified
+                continue
 
-            names = set(fname.split(".")[0] for fname in paths)
+            for filename in sorted(files):
+                paths.append(os.path.join(root, filename))
 
-            # look for __pycache__
-            if os.path.exists(os.path.join(path, "__pycache__")):
-                # add all files from __pycache__ whose filename is not
-                # already in the names we got from the version directory.
-                # add as relative paths including __pycache__ token
-                paths.extend(
-                    os.path.join("__pycache__", pyc)
-                    for pyc in os.listdir(os.path.join(path, "__pycache__"))
-                    if pyc.split(".")[0] not in names
-                )
-            return paths
-        else:
-            return os.listdir(path)
+            if scriptdir.sourceless:
+                # look for __pycache__
+                py_cache_path = os.path.join(root, "__pycache__")
+                if os.path.exists(py_cache_path):
+                    # add all files from __pycache__ whose filename is not
+                    # already in the names we got from the version directory.
+                    # add as relative paths including __pycache__ token
+                    names = {filename.split(".")[0] for filename in files}
+                    paths.extend(
+                        os.path.join(py_cache_path, pyc)
+                        for pyc in os.listdir(py_cache_path)
+                        if pyc.split(".")[0] not in names
+                    )
+
+            if not scriptdir.recursive_version_locations:
+                break
+
+            # the real script order is defined by revision,
+            # but it may be undefined if there are many files with a same
+            # `down_revision`, for a better user experience (ex. debugging),
+            # we use a deterministic order
+            dirs.sort()
+
+        return paths
 
     @classmethod
-    def _from_filename(cls, scriptdir, dir_, filename):
+    def _from_filename(
+        cls, scriptdir: ScriptDirectory, dir_: str, filename: str
+    ) -> Optional[Script]:
         if scriptdir.sourceless:
             py_match = _sourceless_rev_file.match(filename)
         else:
